@@ -76,7 +76,7 @@ setMethod("dim<-", signature(x = "denseMatrix", value = "ANY"),
 		  stop("dim(.) value must be numeric of length 2")
 	      if(prod(dim(x)) != prod(value <- as.integer(value)))
 		  stop("dimensions don't match the number of cells")
-	      clx <- as.character(class(x)) # as.*(): drop attr
+	      clx <- as.character(MatrixClass(class(x))) # as.*(): drop attr
 	      if(substring(clx,2) == "geMatrix") {
 		  x@Dim <- value
 		  if(length(x@factors) > 0)
@@ -123,7 +123,7 @@ setMethod("[", signature(x = "denseMatrix", i = "index", j = "index",
 	      else {
 		  cld <- getClassDef(cl <- class(x))
 		  if(extends(cld, "symmetricMatrix") &&
-		     length(i) == length(j) && all(i == j))
+		     length(i) == length(j) && isTRUE(all(i == j)))
                       ## keep original symmetric class (but not "dpo")
                       as(r, class2(cl, .M.kindC(cld)))
 
@@ -131,16 +131,30 @@ setMethod("[", signature(x = "denseMatrix", i = "index", j = "index",
 	      }
 	  })
 
+.dense.sub.i.2col <- function(x, i, j, ..., drop) {
+    r <- as(x, "matrix")[ i ]
+    if(is.null(dim(r))) r else as(r, geClass(x))
+}
 setMethod("[", signature(x = "denseMatrix", i = "matrix", j = "missing"),#drop="ANY"
-	  function(x, i, j, ..., drop) {
-	      r <- as(x, "matrix")[ i ]
-	      if(is.null(dim(r))) r else as(r, geClass(x))
-	  })
+	  .dense.sub.i.2col)
+setMethod("[", signature(x = "denseMatrix", i = "matrix", j = "missing", drop="missing"),
+	  .dense.sub.i.2col)
+
 
 ## Now the "[<-" ones --- see also those in ./Matrix.R
 ## It's recommended to use setReplaceMethod() rather than setMethod("[<-",.)
 ## even though the former is currently just a wrapper for the latter
 
+## x[] <- value :
+setReplaceMethod("[", signature(x = "denseMatrix", i = "missing", j = "missing",
+				value = "ANY"),## double/logical/...
+	  function (x, value) {
+	      x <- as(x, "generalMatrix")
+	      x@x[] <- value
+	      validObject(x)# check if type and lengths above match
+	      x
+	  })
+
 ## FIXME: 1) These are far from efficient
 ## -----
 setReplaceMethod("[", signature(x = "denseMatrix", i = "index", j = "missing",
@@ -198,10 +212,12 @@ setMethod("isSymmetric", signature(object = "denseMatrix"),
 		  ## test for exact equality; FIXME(?): identical() too strict?
 		  identical(as(object, "lgeMatrix"),
 			    as(t(object), "lgeMatrix"))
-	      else if (is(object, "zMatrix"))
-		  stop("'zMatrix' not yet implemented")
-	      else if (is(object, "iMatrix"))
-		  stop("'iMatrix' not yet implemented")
+	      else if (is(object, "zMatrix")) ## will error out here
+		  identical(as(object, "zgeMatrix"),
+			    as(t(object), "zgeMatrix"))
+	      else if (is(object, "iMatrix")) ## will error out here
+		  identical(as(object, "igeMatrix"),
+			    as(t(object), "igeMatrix"))
 	  })
 
 setMethod("isTriangular", signature(object = "triangularMatrix"),
